function params_opt = MTA_CV_LOOCV(X, numSamplesPerTask, gammas, lambdas, pool_flag)
% This function does leave-one-out to determine best regularization parameters.

errors_tr = zeros(3,length(gammas));
numTasks = length(numSamplesPerTask);

% LOOP ON TASKS.
% If num_tasks is bigger than 50, we'll subsample how many tasks we use for the LOOCV
% otherwise this is very slow. Might be better to parallelize it, but
% a small set of experiments showed no difference.
task_loocv_set = 1:numTasks;
if (length(task_loocv_set) > 50)
    task_loocv_set = 0;
    while(find(task_loocv_set == 0))
        task_loocv_set = floor(numTasks * rand(50,1));
    end
end
for vInd = 1:length(task_loocv_set)
    v = task_loocv_set(vInd);
    
    % We'll hold out one from this task, reducing its num samples:
    if (numSamplesPerTask(v) > 1) % no point in holding out if only 1 sample
        
        for held_out = 1:numSamplesPerTask(v)
            
            clear numSamplesPerTaskOneHeldOut; 
            numSamplesPerTaskOneHeldOut = numSamplesPerTask; % get adjusted number of samples per task
            numSamplesPerTaskOneHeldOut(v) = numSamplesPerTask(v) - 1;
            
            clear X_tr;
            clear held_out_sample;
            X_tr = X; % get adjusted sample matrix
            X_tr{v} = [X{v}(1:(held_out - 1)) X{v}(numSamplesPerTaskOneHeldOut(v):-1:(held_out + 1))];
            held_out_sample = X{v}(held_out);
            
            % Get single task estimates given one held out:
            [mu_st_tr,s_st_tr,~,s_pool_tr] = get_st_and_pooled_stats(X_tr);
            
            if pool_flag == 1; s_st_tr = s_pool_tr; end
            
            % trying all the parameter-dependent estimators
            held_out_err = zeros(3, length(gammas));
            for k = 1:length(gammas)
                % estimated pairwise optimal A
                mu_const_tr = MTA_constant(mu_st_tr, s_st_tr, numSamplesPerTaskOneHeldOut, 'mean', gammas(k)); % constant A
                
                % estimated constant A
                mu_minimax_tr = MTA_constant(mu_st_tr', s_st_tr, numSamplesPerTaskOneHeldOut,'minimax', gammas(k)); % constant A
                
                % average of means regularization
                mu_am_tr = lambdas(k)*mu_st_tr + (1-lambdas(k))*mean(mu_st_tr);
  
                held_out_err(1,k) = held_out_err(1,k) + errors(held_out_sample, mu_const_tr(v));
                held_out_err(2,k) = held_out_err(2,k) + errors(held_out_sample, mu_minimax_tr(v));
                held_out_err(3,k) = held_out_err(3,k) + errors(held_out_sample, mu_am_tr(v));
            end % End loop on cv parameter choices
        end % End loop on held-out samples for this task
        
        task_mean_held_out_error = held_out_err / numSamplesPerTask(v); % divide by number of total held out samples
        errors_tr = errors_tr + task_mean_held_out_error;
        
    end % if there are at least 2 samples so we can hold something out
end % loop on tasks
errors_tr = errors_tr / length(task_loocv_set); % divide by number of tasks estimates made for

% best params
[~,opt_param_inds] = min(errors_tr');

params_opt = [gammas(opt_param_inds(1)) gammas(opt_param_inds(2))...
    lambdas(opt_param_inds(3)) ];


