clear all

%% Choose a dataset to run:

load('shoes.mat')
X = shoes;

%load('espresso.mat') 
%X = espresso;

%load('ml.mat')
%X = ml;

%load('robot_vacuum.mat')
%X = robot_vacuum;

%load repeat_buyers;
%X = repeat_buyers;


% data is a cell with T entries for T tasks
% and the t-th entry is a vector of samples for that task


%% parameters
pool_flag = 1;
gammas = 2.^(-15:1:15);
lambdas = gammas./(gammas+1); % if gamma = lambda/(1-lambda), then lambda = gamma/(1+gamma)
num_estimators = 8; % We compare 8 different estimators.
sum_task_mse = zeros(num_estimators, 1);


for tt = 1:length(X) % iterate through the tasks
    numSamplesPerTask(tt) = length(X{tt});  % get the number of samples per task
end

'NOTE: This is slow because it does a double LOOCV for the cross-validated estimators.'
' Adjust sub-sampling constants in MTA_CV_LOOCV if it is too slow.'

% Hold out the j-th sample of the tt-th task, get all the estimates, 
% compute their error on that 1 held-out sample, repeat.
for tt = 1:length(X)
    
    'Now holding out samples in turn from task' 
    tt
    
    sum_err = zeros(num_estimators, 1);
    for h = 1:numSamplesPerTask(tt) % hold out each sample of the tt-th task in turn
        
        % leave out the j-th sample of the t-th task when estimating:
        held_out_sample = X{tt}(h);
        numSamplesPerTaskOneHeldOut = numSamplesPerTask; % get the adjusted samples-per-task
        numSamplesPerTaskOneHeldOut(tt) = numSamplesPerTask(tt) - 1; 
        A = X{tt}(1:h-1);
        B = X{tt}((h+1):length(X{tt}));
        if (size(A,1) ~= 1) A = A'; end
        if (size(B,1) ~= 1) B = B'; end
        clear Xh;
        Xh = X; % get the adjusted sample matrix
        Xh{tt} = [A B];
        if (size(Xh{tt},1) ~= 1) Xh{tt} = Xh{tt}'; end

        %% Get single-task estimates with one held out.
        [mu_st, s_st,mu_one,s_pool] = get_st_and_pooled_stats(Xh);
        if pool_flag == 1; s_st = s_pool; end
        
        
        %% MTA estimates for one held out.
        [mu_js, mu_const, mu_minimax] = run_MTA_suite(mu_st', s_st, numSamplesPerTaskOneHeldOut');

       
        %% MTA CV estimates - this does LOOCV (on the already-one-held-out-set, 
        %% so this is a double LOOCV) to get the best params
        %% cleanly and independently for each held-out sample. 
        params_opt = MTA_CV_LOOCV(Xh, numSamplesPerTaskOneHeldOut, gammas, lambdas, pool_flag)
        
        % Run standard suite of various convex combinations using the best
        % params determined by the above leave-one-out cv
        [mu_const_CV, mu_minimax_CV, mu_am_CV] = run_MTA_suite_CV(mu_st', s_st', numSamplesPerTaskOneHeldOut, params_opt);
           
        
        %% here are the errors just on this held_out_sample from task tt
        err_st = errors(mu_st(tt), held_out_sample);
        err_one = errors(mu_one, held_out_sample);
        err_js = errors(mu_js(tt), held_out_sample);
        err_am_CV = errors(mu_am_CV(tt), held_out_sample);
        err_const = errors(mu_const(tt), held_out_sample);
        err_const_CV = errors(mu_const_CV(tt), held_out_sample);
        err_minimax = errors(mu_minimax(tt), held_out_sample);
        err_minimax_CV = errors(mu_minimax_CV(tt), held_out_sample);
        
        %% total errors per task
        sum_err(1) = sum_err(1) + err_st;
        sum_err(2) = sum_err(2) + err_one;
        sum_err(3) = sum_err(3) + err_js;
        sum_err(4) = sum_err(4) + err_am_CV;
        sum_err(5) = sum_err(5) + err_const;
        sum_err(6) = sum_err(6) + err_const_CV;
        sum_err(7) = sum_err(7) + err_minimax;
        sum_err(8) = sum_err(8) + err_minimax_CV;
        
    end % End looop on holding out samples
    
    task_mse = sum_err / numSamplesPerTask(tt); 
    sum_task_mse = sum_task_mse + task_mse
end % End loop over tasks

% Convert total_squared_errors to mean_squared_errors for each
% estimator, and put all the MSE for this dataset in one vector:
    mean_task_mse = sum_task_mse / length(X);
    diff_from_single_task = mean_task_mse - repmat(mean_task_mse(1), size(mean_task_mse));
    ratio_change_from_single_task = diff_from_single_task / mean_task_mse(1);


prct = round(ratio_change_from_single_task * 10000) / 100



