function [mu_js,mu_const,mu_minimax,mu_pair,mu_am,mu_pl] = ...
            run_MTA_suite(mu_st,s_st,N,mu_oracle)
        
T = length(mu_st);

simple_flag_js = 1; % james-stein flag
Xi = mean(mu_st);
       
if nargin < 4 

    mu_js = james_stein_diagonal_estimator(mu_st,s_st,Xi,N,simple_flag_js);

    mu_const = MTA_constant(mu_st,s_st,N,'mean'); % constant A
 
    mu_minimax = MTA_constant(mu_st,s_st,N,'minimax'); % minimax A

    if nargout > 3
        
        dists_sq = distsAndSims(mu_st);
        gamma = 1;
        zero_flag = dists_sq == 0;
        dists_sq(zero_flag) = min(min(dists_sq(~zero_flag)));
        A_pair = 2./dists_sq;
        mu_pair = MTA_pairwise(mu_st,s_st,N,gamma,A_pair); % pairwise A
            
    end
    
    if nargout > 4

        energy = sum(dists_sq(:))/2;
        mu_am = AM_reg(mu_st,s_st,N,energy); % average of means

        mu_pl = PL_reg(mu_st,s_st,N); % pooled

    end
    
else % then use oracle values (assuming that s_st that is passed in oracle variance)
    
    mu_js = james_stein_diagonal_estimator(mu_st,s_st,Xi,N,simple_flag_js,mu_oracle);

    dists_sq = distsAndSims(mu_oracle);
    gamma_const = 2*T*(T-1)/sum(dists_sq(:));
    mu_const = MTA_constant(mu_st,s_st,N,'mean',gamma_const); % constant A

    b_hat = max(mu_oracle) - min(mu_oracle);
    gamma_minimax = 2/(b_hat^2);
    mu_minimax = MTA_constant(mu_st,s_st,N,'minimax',gamma_minimax); % minimax A
    
    if nargout > 3
        
        gamma = 1;
        zero_flag = dists_sq == 0;
        dists_sq(zero_flag) = min(min(dists_sq(~zero_flag)));
        A_pair = 2./dists_sq;
        mu_pair = MTA_pairwise(mu_st,s_st,N,gamma,A_pair); % pairwise A
    
    end

    if nargout > 4 

        energy = sum(dists_sq(:))/2;
        mu_am = AM_reg(mu_st,s_st,N,energy); % average of means

        mu_pl = PL_reg(mu_st,s_st,N,mu_oracle); % pooled

    end
end